"""
Pre generation hooks
Currently only synchronizes passwords


{{ cookiecutter.update({"postgres_password": random_ascii_string(20) }) }}
{{ cookiecutter.update({"redis_password": random_ascii_string(20) }) }}
"""